@@ -1,22 +1,10 @@
 /*
-    Copyright (C) 2001, S.R.Haque <srhaque@iee.org>.
-    Copyright (C) 2002, David Faure <david@mandrakesoft.com>
-    Copyright (C) 2004, Arend van Beelen jr. <arend@auton.nl>
     This file is part of the KDE project
+    SPDX-FileCopyrightText: 2001 S .R.Haque <srhaque@iee.org>.
+    SPDX-FileCopyrightText: 2002 David Faure <david@mandrakesoft.com>
+    SPDX-FileCopyrightText: 2004 Arend van Beelen jr. <arend@auton.nl>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2, as published by the Free Software Foundation.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "kfind.h"
@@ -24,17 +12,20 @@
 
 #include "kfinddialog.h"
 
-#include <klocalizedstring.h>
-#include <kmessagebox.h>
-#include <kguiitem.h>
+#include <KGuiItem>
+#include <KLocalizedString>
+#include <KMessageBox>
+
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 70)
+#include <QRegExp>
+#endif
 
 #include <QDialog>
 #include <QDialogButtonBox>
+#include <QHash>
 #include <QLabel>
 #include <QPushButton>
-#include <QtCore/QRegExp>
-#include <QtCore/QHash>
-#include <QTextDocument>
+#include <QRegularExpression>
 #include <QVBoxLayout>
 
 // #define DEBUG_FIND
@@ -43,25 +34,24 @@ static const int INDEX_NOMATCH = -1;
 
 class KFindNextDialog : public QDialog
 {
+    Q_OBJECT
 public:
-    KFindNextDialog(const QString &pattern, QWidget *parent);
+    explicit KFindNextDialog(const QString &pattern, QWidget *parent);
 
     QPushButton *findButton() const;
 
 private:
-    QPushButton *m_findButton;
+    QPushButton *m_findButton = nullptr;
 };
 
 // Create the dialog.
 KFindNextDialog::KFindNextDialog(const QString &pattern, QWidget *parent)
-    : QDialog(parent),
-      m_findButton(0)
+    : QDialog(parent)
 {
     setModal(false);
     setWindowTitle(i18n("Find Next"));
 
-    QVBoxLayout *layout = new QVBoxLayout;
-    setLayout(layout);
+    QVBoxLayout *layout = new QVBoxLayout(this);
 
     layout->addWidget(new QLabel(i18n("<qt>Find next occurrence of '<b>%1</b>'?</qt>", pattern), this));
 
@@ -74,8 +64,8 @@ KFindNextDialog::KFindNextDialog(const QString &pattern, QWidget *parent)
     buttonBox->setStandardButtons(QDialogButtonBox::Close);
     layout->addWidget(buttonBox);
 
-    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
-    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
+    connect(buttonBox, &QDialogButtonBox::accepted, this, &QDialog::accept);
+    connect(buttonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
 }
 
 QPushButton *KFindNextDialog::findButton() const
@@ -86,51 +76,71 @@ QPushButton *KFindNextDialog::findButton() const
 ////
 
 KFind::KFind(const QString &pattern, long options, QWidget *parent)
-    : QObject(parent),
-      d(new KFind::Private(this))
+    : KFind(*new KFindPrivate(this), pattern, options, parent)
 {
+}
+
+KFind::KFind(KFindPrivate &dd, const QString &pattern, long options, QWidget *parent)
+    : QObject(parent)
+    , d(&dd)
+{
+    Q_D(KFind);
+
     d->options = options;
     d->init(pattern);
 }
 
 KFind::KFind(const QString &pattern, long options, QWidget *parent, QWidget *findDialog)
-    : QObject(parent),
-      d(new KFind::Private(this))
+    : KFind(*new KFindPrivate(this), pattern, options, parent, findDialog)
 {
+}
+
+KFind::KFind(KFindPrivate &dd, const QString &pattern, long options, QWidget *parent, QWidget *findDialog)
+    : QObject(parent)
+    , d(&dd)
+{
+    Q_D(KFind);
+
     d->findDialog = findDialog;
     d->options = options;
     d->init(pattern);
 }
 
-void KFind::Private::init(const QString &_pattern)
+void KFindPrivate::init(const QString &_pattern)
 {
+    Q_Q(KFind);
+
     matches = 0;
     pattern = _pattern;
-    dialog = 0;
+    dialog = nullptr;
     dialogClosed = false;
     index = INDEX_NOMATCH;
-    lastResult = NoMatch;
-    regExp = 0;
-    q->setOptions(options);   // create d->regExp with the right options
+    lastResult = KFind::NoMatch;
+
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 70)
+    regExp = nullptr; // QRegExp
+
+#endif
+    // TODO: KF6 change this comment once d->regExp is removed
+    // set options and create d->regExp with the right options
+    q->setOptions(options);
 }
 
-KFind::~KFind()
-{
-    delete d;
-    //qDebug() ;
-}
+KFind::~KFind() = default;
 
 bool KFind::needData() const
 {
+    Q_D(const KFind);
+
     // always true when d->text is empty.
     if (d->options & KFind::FindBackwards)
-        // d->index==-1 and d->lastResult==Match means we haven't answered nomatch yet
-        // This is important in the "replace with a prompt" case.
+    // d->index==-1 and d->lastResult==Match means we haven't answered nomatch yet
+    // This is important in the "replace with a prompt" case.
     {
         return (d->index < 0 && d->lastResult != Match);
     } else
-        // "index over length" test removed: we want to get a nomatch before we set data again
-        // This is important in the "replace with a prompt" case.
+    // "index over length" test removed: we want to get a nomatch before we set data again
+    // This is important in the "replace with a prompt" case.
     {
         return d->index == INDEX_NOMATCH;
     }
@@ -143,6 +153,8 @@ void KFind::setData(const QString &data, int startPos)
 
 void KFind::setData(int id, const QString &data, int startPos)
 {
+    Q_D(KFind);
+
     // cache the data for incremental find
     if (d->options & KFind::FindIncremental) {
         if (id != -1) {
@@ -154,9 +166,9 @@ void KFind::setData(int id, const QString &data, int startPos)
         Q_ASSERT(id <= d->data.size());
 
         if (id == d->data.size()) {
-            d->data.append(Private::Data(id, data, true));
+            d->data.append(KFindPrivate::Data(id, data, true));
         } else {
-            d->data.replace(id, Private::Data(id, data, true));
+            d->data.replace(id, KFindPrivate::Data(id, data, true));
         }
         Q_ASSERT(d->data.at(id).text == data);
     }
@@ -172,7 +184,7 @@ void KFind::setData(int id, const QString &data, int startPos)
             d->index = 0;
         }
 #ifdef DEBUG_FIND
-        //qDebug() << "setData: '" << d->text << "' d->index=" << d->index;
+        // qDebug() << "setData: '" << d->text << "' d->index=" << d->index;
 #endif
         Q_ASSERT(d->index != INDEX_NOMATCH);
         d->lastResult = NoMatch;
@@ -183,10 +195,16 @@ void KFind::setData(int id, const QString &data, int startPos)
 
 QDialog *KFind::findNextDialog(bool create)
 {
+    Q_D(KFind);
+
     if (!d->dialog && create) {
         KFindNextDialog *dialog = new KFindNextDialog(d->pattern, parentWidget());
-        connect(dialog->findButton(), SIGNAL(clicked()), this, SLOT(_k_slotFindNext()));
-        connect(dialog, SIGNAL(finished(int)), this, SLOT(_k_slotDialogClosed()));
+        connect(dialog->findButton(), &QPushButton::clicked, this, [d]() {
+            d->slotFindNext();
+        });
+        connect(dialog, &QDialog::finished, this, [d]() {
+            d->slotDialogClosed();
+        });
         d->dialog = dialog;
     }
     return d->dialog;
@@ -194,6 +212,8 @@ QDialog *KFind::findNextDialog(bool create)
 
 KFind::Result KFind::find()
 {
+    Q_D(KFind);
+
     Q_ASSERT(d->index != INDEX_NOMATCH || d->patternChanged);
 
     if (d->lastResult == Match && !d->patternChanged) {
@@ -214,7 +234,7 @@ KFind::Result KFind::find()
         // if the current pattern is shorter than the matchedPattern we can
         // probably look up the match in the incrementalPath
         if (d->pattern.length() < d->matchedPattern.length()) {
-            Private::Match match;
+            KFindPrivate::Match match;
             if (!d->pattern.isEmpty()) {
                 match = d->incrementalPath.value(d->pattern);
             } else if (d->emptyMatch) {
@@ -226,8 +246,7 @@ KFind::Result KFind::find()
                 bool clean = true;
 
                 // find the first result backwards on the path that isn't dirty
-                while (d->data.at(match.dataId).dirty == true &&
-                        !d->pattern.isEmpty()) {
+                while (d->data.at(match.dataId).dirty == true && !d->pattern.isEmpty()) {
                     d->pattern.truncate(d->pattern.length() - 1);
 
                     match = d->incrementalPath.value(d->pattern);
@@ -250,9 +269,15 @@ KFind::Result KFind::find()
                 // if the result is clean we can return it now
                 if (clean) {
                     if (d->customIds) {
-                        emit highlight(d->currentId, d->index, d->matchedLength);
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 81)
+                        Q_EMIT highlight(d->currentId, d->index, d->matchedLength);
+#endif
+                        Q_EMIT textFoundAtId(d->currentId, d->index, d->matchedLength);
                     } else {
-                        emit highlight(d->text, d->index, d->matchedLength);
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 81)
+                        Q_EMIT highlight(d->text, d->index, d->matchedLength);
+#endif
+                        Q_EMIT textFound(d->text, d->index, d->matchedLength);
                     }
 
                     d->lastResult = Match;
@@ -294,18 +319,14 @@ KFind::Result KFind::find()
     }
 
 #ifdef DEBUG_FIND
-    //qDebug() << "d->index=" << d->index;
+    // qDebug() << "d->index=" << d->index;
 #endif
     do {
         // if we have multiple data blocks in our cache, walk through these
         // blocks till we either searched all blocks or we find a match
         do {
             // Find the next candidate match.
-            if (d->options & KFind::RegularExpression) {
-                d->index = KFind::find(d->text, *d->regExp, d->index, d->options, &d->matchedLength);
-            } else {
-                d->index = KFind::find(d->text, d->pattern, d->index, d->options, &d->matchedLength);
-            }
+            d->index = KFind::find(d->text, d->pattern, d->index, d->options, &d->matchedLength, nullptr);
 
             if (d->options & KFind::FindIncremental) {
                 d->data[d->currentId].dirty = false;
@@ -332,13 +353,13 @@ KFind::Result KFind::find()
                 if (d->options & KFind::FindIncremental) {
                     if (d->pattern.isEmpty()) {
                         delete d->emptyMatch;
-                        d->emptyMatch = new Private::Match(d->currentId, d->index, d->matchedLength);
+                        d->emptyMatch = new KFindPrivate::Match(d->currentId, d->index, d->matchedLength);
                     } else {
-                        d->incrementalPath.insert(d->pattern, Private::Match(d->currentId, d->index, d->matchedLength));
+                        d->incrementalPath.insert(d->pattern, KFindPrivate::Match(d->currentId, d->index, d->matchedLength));
                     }
 
                     if (d->pattern.length() < d->matchedPattern.length()) {
-                        d->pattern += d->matchedPattern.mid(d->pattern.length(), 1);
+                        d->pattern += QStringView(d->matchedPattern).mid(d->pattern.length(), 1);
                         done = false;
                     }
                 }
@@ -348,9 +369,15 @@ KFind::Result KFind::find()
                     // Tell the world about the match we found, in case someone wants to
                     // highlight it.
                     if (d->customIds) {
-                        emit highlight(d->currentId, d->index, d->matchedLength);
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 81)
+                        Q_EMIT highlight(d->currentId, d->index, d->matchedLength);
+#endif
+                        Q_EMIT textFoundAtId(d->currentId, d->index, d->matchedLength);
                     } else {
-                        emit highlight(d->text, d->index, d->matchedLength);
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 81)
+                        Q_EMIT highlight(d->text, d->index, d->matchedLength);
+#endif
+                        Q_EMIT textFound(d->text, d->index, d->matchedLength);
                     }
 
                     if (!d->dialogClosed) {
@@ -358,7 +385,7 @@ KFind::Result KFind::find()
                     }
 
 #ifdef DEBUG_FIND
-                    //qDebug() << "Match. Next d->index=" << d->index;
+                    // qDebug() << "Match. Next d->index=" << d->index;
 #endif
                     d->lastResult = Match;
                     return Match;
@@ -383,16 +410,16 @@ KFind::Result KFind::find()
     } while (d->index != INDEX_NOMATCH);
 
 #ifdef DEBUG_FIND
-    //qDebug() << "NoMatch. d->index=" << d->index;
+    // qDebug() << "NoMatch. d->index=" << d->index;
 #endif
     d->lastResult = NoMatch;
     return NoMatch;
 }
 
-void KFind::Private::startNewIncrementalSearch()
+void KFindPrivate::startNewIncrementalSearch()
 {
-    Private::Match *match = emptyMatch;
-    if (match == 0) {
+    KFindPrivate::Match *match = emptyMatch;
+    if (match == nullptr) {
         text.clear();
         index = 0;
         currentId = 0;
@@ -403,7 +430,8 @@ void KFind::Private::startNewIncrementalSearch()
     }
     matchedLength = 0;
     incrementalPath.clear();
-    delete emptyMatch; emptyMatch = 0;
+    delete emptyMatch;
+    emptyMatch = nullptr;
     matchedPattern = pattern;
     pattern.clear();
 }
@@ -438,15 +466,59 @@ static bool matchOk(const QString &text, int index, int matchedLength, long opti
     return false;
 }
 
+static int findRegex(const QString &text, const QString &pattern, int index, long options, int *matchedLength, QRegularExpressionMatch *rmatch)
+{
+    QString _pattern = pattern;
+
+    // Always enable Unicode support in QRegularExpression
+    QRegularExpression::PatternOptions opts = QRegularExpression::UseUnicodePropertiesOption;
+    // instead of this rudimentary test, add a checkbox to toggle MultilineOption ?
+    if (pattern.startsWith(QLatin1Char('^')) || pattern.endsWith(QLatin1Char('$'))) {
+        opts |= QRegularExpression::MultilineOption;
+    } else if (options & KFind::WholeWordsOnly) { // WholeWordsOnly makes no sense with multiline
+        _pattern = QLatin1String("\\b") + pattern + QLatin1String("\\b");
+    }
+
+    if (!(options & KFind::CaseSensitive)) {
+        opts |= QRegularExpression::CaseInsensitiveOption;
+    }
+
+    QRegularExpression re(_pattern, opts);
+    QRegularExpressionMatch match;
+    if (options & KFind::FindBackwards) {
+        // Backward search, until the beginning of the line...
+        text.lastIndexOf(re, index, &match);
+    } else {
+        // Forward search, until the end of the line...
+        match = re.match(text, index);
+    }
+
+    // index is -1 if no match is found
+    index = match.capturedStart(0);
+    // matchedLength is 0 if no match is found
+    *matchedLength = match.capturedLength(0);
+
+    if (rmatch) {
+        *rmatch = match;
+    }
+
+    return index;
+}
+
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 70)
 // static
 int KFind::find(const QString &text, const QString &pattern, int index, long options, int *matchedLength)
+{
+    return find(text, pattern, index, options, matchedLength, nullptr);
+}
+#endif
+
+// static
+int KFind::find(const QString &text, const QString &pattern, int index, long options, int *matchedLength, QRegularExpressionMatch *rmatch)
 {
     // Handle regular expressions in the appropriate way.
     if (options & KFind::RegularExpression) {
-        Qt::CaseSensitivity caseSensitive = (options & KFind::CaseSensitive) ? Qt::CaseSensitive : Qt::CaseInsensitive;
-        QRegExp regExp(pattern, caseSensitive);
-
-        return find(text, regExp, index, options, matchedLength);
+        return findRegex(text, pattern, index, options, matchedLength, rmatch);
     }
 
     // In Qt4 QString("aaaaaa").lastIndexOf("a",6) returns -1; we need
@@ -470,7 +542,7 @@ int KFind::find(const QString &text, const QString &pattern, int index, long opt
                 break;
             }
             index--;
-            //qDebug() << "decrementing:" << index;
+            // qDebug() << "decrementing:" << index;
         }
     } else {
         // Forward search, until the end of the line...
@@ -487,7 +559,7 @@ int KFind::find(const QString &text, const QString &pattern, int index, long opt
             index++;
         }
         if (index > text.length()) { // end of line
-            //qDebug() << "at" << index << "-> not found";
+            // qDebug() << "at" << index << "-> not found";
             index = -1; // not found
         }
     }
@@ -499,6 +571,7 @@ int KFind::find(const QString &text, const QString &pattern, int index, long opt
     return index;
 }
 
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 70)
 // Core method for the regexp-based find
 static int doFind(const QString &text, const QRegExp &pattern, int index, long options, int *matchedLength)
 {
@@ -511,7 +584,7 @@ static int doFind(const QString &text, const QRegExp &pattern, int index, long o
                 break;
             }
 
-            /*int pos =*/ pattern.indexIn(text.mid(index));
+            /*int pos =*/pattern.indexIn(text.mid(index));
             *matchedLength = pattern.matchedLength();
             if (matchOk(text, index, *matchedLength, options)) {
                 break;
@@ -527,7 +600,7 @@ static int doFind(const QString &text, const QRegExp &pattern, int index, long o
                 break;
             }
 
-            /*int pos =*/ pattern.indexIn(text.mid(index));
+            /*int pos =*/pattern.indexIn(text.mid(index));
             *matchedLength = pattern.matchedLength();
             if (matchOk(text, index, *matchedLength, options)) {
                 break;
@@ -543,7 +616,9 @@ static int doFind(const QString &text, const QRegExp &pattern, int index, long o
     }
     return index;
 }
+#endif
 
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 70)
 // Since QRegExp doesn't support multiline searches (the equivalent of perl's /m)
 // we have to cut the text into lines if the pattern starts with ^ or ends with $.
 static int lineBasedFind(const QString &text, const QRegExp &pattern, int index, long options, int *matchedLength)
@@ -561,7 +636,6 @@ static int lineBasedFind(const QString &text, const QRegExp &pattern, int index,
     }
 
     if (options & KFind::FindBackwards) {
-
         if (startLineNumber == lines.count()) {
             // We went too far, go back to the last line
             --startLineNumber;
@@ -589,7 +663,9 @@ static int lineBasedFind(const QString &text, const QRegExp &pattern, int index,
     }
     return -1;
 }
+#endif
 
+#if KTEXTWIDGETS_ENABLE_DEPRECATED_SINCE(5, 70)
 // static
 int KFind::find(const QString &text, const QRegExp &pattern, int index, long options, int *matchedLength)
 {
@@ -599,27 +675,33 @@ int KFind::find(const QString &text, const QRegExp &pattern, int index, long opt
 
     return doFind(text, pattern, index, options, matchedLength);
 }
+#endif
 
-void KFind::Private::_k_slotFindNext()
+void KFindPrivate::slotFindNext()
 {
-    emit q->findNext();
+    Q_Q(KFind);
+
+    Q_EMIT q->findNext();
 }
 
-void KFind::Private::_k_slotDialogClosed()
+void KFindPrivate::slotDialogClosed()
 {
+    Q_Q(KFind);
+
 #ifdef DEBUG_FIND
-    //qDebug() << " Begin";
+    // qDebug() << " Begin";
 #endif
-    emit q->dialogClosed();
+    Q_EMIT q->dialogClosed();
     dialogClosed = true;
 #ifdef DEBUG_FIND
-    //qDebug() << " End";
+    // qDebug() << " End";
 #endif
-
 }
 
 void KFind::displayFinalDialog() const
 {
+    Q_D(const KFind);
+
     QString message;
     if (numMatches()) {
         message = i18np("1 match found.", "%1 matches found.", numMatches());
@@ -631,6 +713,8 @@ void KFind::displayFinalDialog() const
 
 bool KFind::shouldRestart(bool forceAsking, bool showNumMatches) const
 {
+    Q_D(const KFind);
+
     // Only ask if we did a "find from cursor", otherwise it's pointless.
     // Well, unless the user can modify the document during a search operation,
     // hence the force boolean.
@@ -655,74 +739,96 @@ bool KFind::shouldRestart(bool forceAsking, bool showNumMatches) const
 
     message += QLatin1String("<br><br>"); // can't be in the i18n() of the first if() because of the plural form.
     // Hope this word puzzle is ok, it's a different sentence
-    message +=
-        (d->options & KFind::FindBackwards) ?
-        i18n("Continue from the end?")
-        : i18n("Continue from the beginning?");
+    message += (d->options & KFind::FindBackwards) ? i18n("Continue from the end?") : i18n("Continue from the beginning?");
 
-    int ret = KMessageBox::questionYesNo(dialogsParent(), QString::fromLatin1("<qt>%1</qt>").arg(message),
-                                         QString(), KStandardGuiItem::cont(), KStandardGuiItem::stop());
-    bool yes = (ret == KMessageBox::Yes);
+    int ret = KMessageBox::questionTwoActions(dialogsParent(),
+                                              QStringLiteral("<qt>%1</qt>").arg(message),
+                                              QString(),
+                                              KStandardGuiItem::cont(),
+                                              KStandardGuiItem::stop());
+    bool yes = (ret == KMessageBox::PrimaryAction);
     if (yes) {
-        const_cast<KFind *>(this)->d->options &= ~KFind::FromCursor;    // clear FromCursor option
+        const_cast<KFindPrivate *>(d)->options &= ~KFind::FromCursor; // clear FromCursor option
     }
     return yes;
 }
 
 long KFind::options() const
 {
+    Q_D(const KFind);
+
     return d->options;
 }
 
 void KFind::setOptions(long options)
 {
+    Q_D(KFind);
+
     d->options = options;
 
+#if KTEXTWIDGETS_BUILD_DEPRECATED_SINCE(5, 70)
     delete d->regExp;
     if (d->options & KFind::RegularExpression) {
         Qt::CaseSensitivity caseSensitive = (d->options & KFind::CaseSensitive) ? Qt::CaseSensitive : Qt::CaseInsensitive;
         d->regExp = new QRegExp(d->pattern, caseSensitive);
     } else {
-        d->regExp = 0;
+        d->regExp = nullptr;
     }
+#endif
 }
 
 void KFind::closeFindNextDialog()
 {
+    Q_D(KFind);
+
     if (d->dialog) {
         d->dialog->deleteLater();
-        d->dialog = 0;
+        d->dialog = nullptr;
     }
     d->dialogClosed = true;
 }
 
 int KFind::index() const
 {
+    Q_D(const KFind);
+
     return d->index;
 }
 
 QString KFind::pattern() const
 {
+    Q_D(const KFind);
+
     return d->pattern;
 }
 
 void KFind::setPattern(const QString &pattern)
 {
-    if (d->options & KFind::FindIncremental && d->pattern != pattern) {
+    Q_D(KFind);
+
+    if (d->pattern != pattern) {
         d->patternChanged = true;
+        d->matches = 0;
     }
 
     d->pattern = pattern;
-    setOptions(options());   // rebuild d->regExp if necessary
+
+    // TODO: KF6 change this comment once d->regExp is removed
+    // set the options and rebuild d->regeExp if necessary
+    setOptions(options());
 }
 
 int KFind::numMatches() const
 {
+    Q_D(const KFind);
+
     return d->matches;
 }
 
 void KFind::resetCounts()
 {
+    Q_D(KFind);
+
     d->matches = 0;
 }
 
@@ -733,15 +839,18 @@ bool KFind::validateMatch(const QString &, int, int)
 
 QWidget *KFind::parentWidget() const
 {
-    return (QWidget *)parent();
+    return static_cast<QWidget *>(parent());
 }
 
 QWidget *KFind::dialogsParent() const
 {
+    Q_D(const KFind);
+
     // If the find dialog is still up, it should get the focus when closing a message box
     // Otherwise, maybe the "find next?" dialog is up
     // Otherwise, the "view" is the parent.
-    return d->findDialog ? (QWidget *)d->findDialog : (d->dialog ? d->dialog : parentWidget());
+    return d->findDialog ? static_cast<QWidget *>(d->findDialog) : (d->dialog ? d->dialog : parentWidget());
 }
 
+#include "kfind.moc"
 #include "moc_kfind.cpp"
